var express = require('express');
var path = require('path');
var winston = require('winston');
var logger = winston.loggers.get('default');
var moment = require('moment');

module.exports = function(app, passport, models) {

    /**
     * route to create a web-only user
     */
    app.post('/api/signup', passport.authenticate('signup', {
        successRedirect: '/index',
        failureRedirect: '/'
    }));

    /**5:44
     * route to test if the user is logged in or not
     */
    app.get('/api/loggedin/', function(req, res) {
        res.send(req.isAuthenticated() ? req.user : '0');
    });

    /**
     * route to log in
     */
    app.post('/api/login', function(req, res, next) {
        passport.authenticate('local', function(err, user) {
            if (err) {
                logger.error("login error: " + err);
                return next(err);}
            if (!user) {
                logger.error("no user found");

                return next("No User Found. Please Try Again.");
            }
            logger.debug("logging in...");
            //now use passport to actually log in
            req.login(user, function(err) {

                    if (err) {
                        logger.error("req.login error: " + err);

                        return next(err);
                    }
                    logger.debug("req.login user: " + req.user);

                    res.send(req.user);
                }
            );
        })(req, res, next);
    });

    /**
     * route to log out
     */
    app.post('/api/logout', function(req, res){
        req.logOut();
        res.send(200);
    });

    /**
     * Enforce authentication on all routes.
     * If the main index page is requested, send back the countData page
     */
    app.get('*', auth, function (req, res, next) {
        res.sendFile('components/public/countData/countData.html', {root: path.join(__dirname, '..')});
    });

    /**************    Helpers for Web Application**********************/


    /**
     * Common routine for external filtering the web grids
     */
    function buildWhereArgs(req, objName, objRawAttributes) {
        var whereArgs = [];
        if (req.body.where != null) {
            console.log(JSON.stringify(req.body.where));
            for (var key in req.body.where) {
                if (objRawAttributes.hasOwnProperty(key)) {
                    var newCondition = String(req.body.where[key]);
                   if (newCondition === "[object Object]") {
                        if (key === "timestamp") {
                            //we have gt/lt range
                            if (req.body.where[key]['$gt']) {
                                var newWhere = {};
                                newWhere[objRawAttributes[key].field] = {$gt: moment(req.body.where[key]['$gt'])};
                                whereArgs.push(newWhere);
                            }
                            if (req.body.where[key]['$lt']) {
                                var newWhere = {};
                                newWhere[objRawAttributes[key].field] = {$lt: moment(req.body.where[key]['$lt'])};
                                whereArgs.push(newWhere);
                            }
                            continue;
                        } else {
                            newCondition = String(req.body.where[key]['$like']);
                        }
                    }
                    var dataType = String(objRawAttributes[key].type);
                    if (newCondition.indexOf("*") < 0) {
                        newCondition = "%" + newCondition + "%";
                    } else {
                        newCondition = newCondition.replace(/\*/g, "%");
                    }
                    if (dataType.indexOf("VARCHAR") < 0) {
                        //cast numerics as strings so we can use the "LIKE" operator
                        if (dataType.indexOf("TIME") > 0) {
                            //since time is hidden in the UI, we should strip time from the date fields
                            whereArgs.push(models.Sequelize.where(models.Sequelize.cast(models.Sequelize.cast(models.Sequelize.col(objName + "." + objRawAttributes[key].field), 'DATE'), 'varchar'), 'LIKE', newCondition));
                        } else {
                            whereArgs.push(models.Sequelize.where(models.Sequelize.cast(models.Sequelize.col(objName + "." + objRawAttributes[key].field), 'varchar'), 'LIKE', newCondition));
                        }
                        delete req.body.where[key];
                    } else {
                        //already a String... just use the LIKE operator
                        var newWhere = {};

                        newWhere[objRawAttributes[key].field] = {$like: newCondition};
                        whereArgs.push(newWhere);
                    }
                }
            };
        }
        return whereArgs;
    }
    //CountData Grid
    function buildCountDataWhereArgs(req) {
        var args = buildWhereArgs(req, 'CountData', models.CountData.rawAttributes);
        var newWhere = {};
        newWhere[models.CountData.rawAttributes['deviceId'].field] = {$ne : null };
        args.push(newWhere);
        return args;
    }
    //Inventory/PO/Receipt/Consumption Grids
    function buildInvWhereArgs(req) {
        return buildWhereArgs(req, 'Inventory', models.Inventory.rawAttributes);
    }
    //FTP Grid
    function buildFTPWhereArgs(req) {
        return buildWhereArgs(req, 'FTPLog', models.FTPLog.rawAttributes);
    }
    //Login Messages Grid
    function buildMessageWhereArgs(req) {
        return buildWhereArgs(req, 'Message', models.Message.rawAttributes);
    }
    //NDC (POC) Grid
    function buildNdcWhereArgs(req) {
        return buildWhereArgs(req, 'NdcData', models.NdcData.rawAttributes);
    }


    /************************* APIs for Web Application *************************/

    /**
     * Return a total count of Count Data items that meet the filter criteria
     * (for pagination)
     *
     * Params:  (Body) "where" object with attributes that match the CountData model
     */
    app.post('/api/getCountDataTotal/', function(req, res) {
        var whereArgs = buildCountDataWhereArgs(req);

        models.CountData.count({
                where: whereArgs,
                limit: req.body.limit,
                offset: req.body.offset
            }
        ).then(function(count) {
            res.send(String(count));
        })
    });

    /**
     * Return the actual data that meets the filter criteria
     *
     * Params:  (Body) "where" object with attributes that match the CountData model
     */
    app.post('/api/getCountData/', function(req, res) {
        var whereArgs = buildCountDataWhereArgs(req);


        models.CountData.findAll(
            {
                where: whereArgs,
                limit: req.body.limit,
                offset: req.body.offset,
                order: req.body.order
            }

        ).then(function (countData) {
            res.send(countData);
        });
    });

    /**
     * Return a total count of On Hand Inventory Data items that meet the where criteria
     * (for pagination)
     *
     * Params:  (Body) "where" object with attributes that match the Inventory model
     */
    app.post('/api/getInventoryTotal/', function(req, res) {
        var whereArgs = buildInvWhereArgs(req);
        whereArgs.push({type: 'INV'});

        models.Inventory.count({
                where: whereArgs,
                limit: req.body.limit,
                offset: req.body.offset
            }
        ).then(function(count) {
            res.send(String(count));
        })
    });

    /**
     * Return all On Hand Inventory data that meets the filter criteria
     *
     * Params:  (Body) "where" object with attributes that match the Inventory model
     */
    app.post('/api/getInventory/', function(req, res) {
        var whereArgs = buildInvWhereArgs(req);
        whereArgs.push({type: 'INV'});

        models.Inventory.findAll(
            {
                where: whereArgs,
                limit: req.body.limit,
                offset: req.body.offset,
                order: req.body.order
            }

        ).then(function (inventory) {
            res.send(inventory);
        });
    });

    /**
     * Return a total count of PO Data items that meet the where criteria
     * (for pagination)
     *
     * Params:  (Body) "where" object with attributes that match the Inventory model
     */
    app.post('/api/getPoTotal/', function(req, res) {
        var whereArgs = buildInvWhereArgs(req);
        whereArgs.push({type: 'PO'});

        models.Inventory.count({
                where: whereArgs,
                limit: req.body.limit,
                offset: req.body.offset
            }
        ).then(function(count) {
            res.send(String(count));
        })
    });

    /**
     * Return all PO data that meets the filter criteria
     *
     * Params:  (Body) "where" object with attributes that match the Inventory model
     */
    app.post('/api/getPo/', function(req, res) {
        var whereArgs = buildInvWhereArgs(req);
        whereArgs.push({type: 'PO'});
        models.Inventory.findAll(
            {
                where: whereArgs,
                limit: req.body.limit,
                offset: req.body.offset,
                order: req.body.order
            }

        ).then(function (inventory) {
            res.send(inventory);
        });
    });

    /**
     * Return a total count of Receipts Data items that meet the where criteria
     * (for pagination)
     *
     * Params:  (Body) "where" object with attributes that match the Inventory model
     */
    app.post('/api/getReceiptTotal/', function(req, res) {
        var whereArgs = buildInvWhereArgs(req);
        whereArgs.push({type: 'RECEIPT'});

        models.Inventory.count({
                where: whereArgs,
                limit: req.body.limit,
                offset: req.body.offset
            }
        ).then(function(count) {
            res.send(String(count));
        })
    });

    /**
     * Return all Receipt data that meets the filter criteria
     *
     * Params:  (Body) "where" object with attributes that match the Inventory model
     */
    app.post('/api/getReceipt/', function(req, res) {
        var whereArgs = buildInvWhereArgs(req);
        whereArgs.push({type: 'RECEIPT'});

        models.Inventory.findAll(
            {
                where: whereArgs,
                limit: req.body.limit,
                offset: req.body.offset,
                order: req.body.order
            }

        ).then(function (receipt) {
            res.send(receipt);
        });
    });

    /**
     * Return a total count of Consumption Data items that meet the where criteria
     * (for pagination)
     *
     * Params:  (Body) "where" object with attributes that match the Inventory model
     */
    app.post('/api/getConsumptionTotal/', function(req, res) {
        var whereArgs = buildInvWhereArgs(req);
        whereArgs.push({type: 'CONSUMERUSAGE'});

        models.Inventory.count({
                where: whereArgs,
                limit: req.body.limit,
                offset: req.body.offset
            }
        ).then(function(count) {
            res.send(String(count));
        })
    });


    /**
     * Return all Consumption data that meets the filter criteria
     *
     * Params:  (Body) "where" object with attributes that match the Inventory model
     */
    app.post('/api/getConsumption/', function(req, res) {
        var whereArgs = buildInvWhereArgs(req);
        whereArgs.push({type: 'CONSUMERUSAGE'});
        models.Inventory.findAll(
            {
                where: whereArgs,
                limit: req.body.limit,
                offset: req.body.offset,
                order: req.body.order
            }

        ).then(function (consumption) {
            res.send(consumption);
        });
    });

    /**
     * Used for UI or REST console to list FTP log entries
     *
     * Params:  (Body) "where" object with attributes that match the FTP model
     */
    app.post('/api/getFTPdata', function(req, res) {
        var whereArgs = buildFTPWhereArgs(req);
        var ob = req.body.order;
        if (!ob) {
            ob = '"sent" DESC';
        }
        models.FTPLog.findAll(
            {
                where: whereArgs,
                limit: req.body.limit,
                offset: req.body.offset,
                order: ob
            }
        ).then(function(logs) {
            res.send(logs);
        }).catch(function(err){
            console.log(err);
            res.sendStatus(500);
        });
    });

    /**
     * Return a total count of FTP Data items that meet the where criteria
     * (for pagination)
     *
     * Params:  (Body) "where" object with attributes that match the FTP model
     */
    app.post('/api/getFTPDataTotal', function(req, res) {
        var whereArgs = buildFTPWhereArgs(req);
        var ob = req.body.order;
        if (!ob) {
            ob = '"sent" DESC';
        }
        models.FTPLog.count(
            {
                where: whereArgs,
                limit: req.body.limit,
                offset: req.body.offset,
                order: ob
            }
        ).then(function(logs) {
            res.send(logs);
        }).catch(function(err){
            console.log(err);
            res.sendStatus(500);
        });
    });

    /**
     * Returns all Vendors in the table
     */
    app.post('/api/getVendorList', function(req, res) {
        models.VendorList.findAll({order: '"vendorDisplay" ASC'}).then(function(vendors) {
            res.send(vendors);
        });
    });

    /**
     * Return All OR a specific Vendor
     *
     * Optional Params:  (Body) vendorId
     */
    app.post('/api/getVendor', function(req, res) {
        if (req.body.vendorId == null) {
            models.VendorList.findAll()
                .then(function (items) {
                    res.send(items);
                });
        } else {
            models.VendorList.findAll({
                where: {
                    $or: [
                        {vendorId: req.body.vendorId},
                        {vendorDisplay: req.body.vendorId}
                    ]
                }
            }).then(function (items) {
                res.send(items);
            });
        }
    });

    /**
     * Return a total count of Login message Data items that meet the filter criteria
     * (for pagination)
     *
     * Params:  (Body) "where" object with attributes that match the Message model
     */
    app.post('/api/getMessagesTotal/', function(req, res) {
        var whereArgs = buildMessageWhereArgs(req);

        models.Message.count({
                where: whereArgs,
                limit: req.body.limit,
                offset: req.body.offset
            }
        ).then(function(count) {
            res.send(String(count));
        })
    });

    /**
     * Return all messages that meet the filter criteria
     *
     * Params:  (Body) "where" object with attributes that match the Message model
     */
    app.post('/api/getMessages/', function(req, res) {
        var whereArgs = buildMessageWhereArgs(req);


        models.Message.findAll(
            {
                where: whereArgs,
                include: [
                    {model: models.User, as: "createdBy", attributes: ['displayName']},
                    {model: models.MsgRecipient, as: "recipients", include: [ {model: models.ClinicProfile, as: "clinic", attributes:['clinicId']}]}
                    ],
                limit: req.body.limit,
                offset: req.body.offset,
                order: req.body.order
            }

        ).then(function (mData) {
            res.send(mData);
        });
    });

    /**
     * Returns all clinics in the table.  Used for Message maintenance.
     */
    app.post('/api/getAllClinics', function(req,res){
       models.ClinicProfile.findAll({
           attributes: ['id', 'clinicId']
       }).then(function(data) {
           res.send(data);
       }).catch(function(err){
           res.status(500).send(err);
       })
    });

    /**
     * Return the count of NDC (POC) data that meet the filter criteria
     * (for pagination)
     *
     * Params:  (Body) "where" object with attributes that match the NDC model
     */
    app.post('/api/getNdcDataTotal/', function(req, res) {
        var whereArgs = buildNdcWhereArgs(req);

        models.NdcData.count({
                where: whereArgs,
                limit: req.body.limit,
                offset: req.body.offset
            }
        ).then(function(count) {
            res.send(String(count));
        })
    });

    /**
     * Return all NDC (POC) data that meet the filter criteria
     *
     * Params:  (Body) "where" object with attributes that match the NDC model
     */
    app.post('/api/getNdcData/', function(req, res) {
        var whereArgs = buildNdcWhereArgs(req);


        models.NdcData.findAll(
            {
                where: whereArgs,
                limit: req.body.limit,
                offset: req.body.offset,
                order: req.body.order
            }

        ).then(function (ndcData) {
            res.send(ndcData);
        });
    });

};

/**
 * Require authentication for ALL methods except GET on favicon.
 * @param req Request object
 * @param res Response object
 * @param next chained callback
 */
var auth = function(req, res, next) {
    if (req.url == "assets/images/favicon.png" || req.url == "/favicon.ico") {
        next();
    }
    else if (!req.isAuthenticated())
        res.send(401, 'Please log in');
    else next();
};



